#pragma once

/**
* Class Music
* This class represents a single music. 
* @author Adrian Gaudebert - adrian.gaudebert@gmail.com
* @author Simon Stephan - s.stephan88@gmail.com
*/
class Music
{

private:	//--> Attributs privs

	sf::Music m_Music;

	PausableClock m_Timer;

	CString m_FileName;

public:		//--> Mthodes publiques

	/**
	* Default constructor
	*/
	Music();

	/**
	* Load a music from a file
	* @param _FileName Link to the file, MFC CString
	*/
	bool Load(const CString& _FileName);

	/**
	* Load a music from a file
	* @param _FileName Link to the file, C string
	*/
	bool Load(const char* _FileName);

	/**
	* Play the music
	*/
	void Play();

	/**
	* Stop the music
	*/
	void Stop();

	/**
	* Check wether the music is stopped
	*/
	bool Stopped() const;

	/**
	* Check wether the music is paused
	*/
	bool Paused() const;

	/**
	* Check wether the music is playing
	*/
	bool Playing() const;

	/**
	* Check wether the music has reach its end
	*/
	bool Ended();

	/**
	* Get the file name of the music
	* @return Link to the file
	*/
	const CString& GetFileName() const;

	/**
	* Set the volume of the music
	* @param _Volume Float between 0 and 100
	*/
	void SetVolume(float _Volume);

};
