#include "StdAfx.h"
#include "saMP.h"

IMPLEMENT_SERIAL(PlayList, CObject, 1)

/* ------------------ */
/* PlayList::PlayList */
/* ------------------ */
PlayList::PlayList()
{
	m_CurrentPosition = m_MusicList.GetHeadPosition();
	AudioConfig::Get().AddObserver(this);
}

/* ------------------- */
/* PlayList::~PlayList */
/* ------------------- */
PlayList::~PlayList()
{
	ClearList();
}

/* ---------------- */
/* PlayList::Manage */
/* ---------------- */
void PlayList::Manage()
{
	CSingleLock Lock( &AccessMusicList );
	Lock.Lock();

	Music *ptrMusic = GetCurrent();

	// On regarde si la chanson en cours est termine
	if ( ptrMusic != NULL )
	{
		if ( ptrMusic->Ended() )
		{
			Next();

			if ( GetCurrent()->Stopped() )
			{
				// On est arriv  la fin de la liste de lecture et le mode repeat n'est pas activ
				m_CurrentPosition = m_MusicList.GetHeadPosition();
			}
		}
	}

	Lock.Unlock();
}

/* -------------------- */
/* PlayList::GetCurrent */
/* -------------------- */
Music* PlayList::GetCurrent()
{
	if (m_CurrentPosition == NULL)
	{
		if ( !m_MusicList.IsEmpty() )
		{
			m_CurrentPosition = m_MusicList.GetHeadPosition();
		}
		else
		{
			return NULL;
		}
	}
	return ( m_MusicList.GetAt(m_CurrentPosition) );
}

/* ----------------- */
/* PlayList::GetLast */
/* ----------------- */
Music* PlayList::GetLast()
{
	if( m_MusicList.IsEmpty() )
	{
		return NULL;
	}
	return ( m_MusicList.GetTail() );
}

/* -------------- */
/* PlayList::Next */
/* -------------- */
void PlayList::Next(bool _Force)
{
	// Si au moins une musique est charge
	if (GetCurrent() != NULL)
	{
		CSingleLock Lock( &AccessMusicList );
		Lock.Lock();

		GetCurrent()->Stop();

		// Si la lecture alatoire est active
		if ( AudioConfig::Get().GetRandom() )
		{
			m_CurrentPosition = GetRandomIterator();
			m_MusicList.GetAt(m_CurrentPosition)->Play();
		}
		else
		{
			// Si on est  la fin de la playlist
			if ( m_CurrentPosition == m_MusicList.GetTailPosition() )
			{
				// Si l'utilisateur clique sur le bouton ou si le mode repeat est activ
				if ( _Force || AudioConfig::Get().GetRepeat() )
				{
					// On revient au dbut
					m_CurrentPosition = m_MusicList.GetHeadPosition();
					m_MusicList.GetAt(m_CurrentPosition)->Play();
				}
			}
			else
			{
				// Comportement normal, passage  la musique suivante
				m_MusicList.GetNext(m_CurrentPosition);
				m_MusicList.GetAt(m_CurrentPosition)->Play();
			}
		}
		m_MusicList.GetAt(m_CurrentPosition)->SetVolume( AudioConfig::Get().GetVolume() );

		Lock.Unlock();
	}
}

/* ------------------ */
/* PlayList::Previous */
/* ------------------ */
void PlayList::Previous()
{
	if (GetCurrent() != NULL)
	{
		CSingleLock Lock( &AccessMusicList );
		Lock.Lock();

		GetCurrent()->Stop();
		if ( AudioConfig::Get().GetRandom() )
		{
			m_CurrentPosition = GetRandomIterator();
			m_MusicList.GetAt(m_CurrentPosition)->Play();
		}
		else
		{
			if ( m_CurrentPosition == m_MusicList.GetHeadPosition() )
			{
				m_CurrentPosition = m_MusicList.GetTailPosition();
			}
			else
			{
				m_MusicList.GetPrev(m_CurrentPosition);
			}
			m_MusicList.GetAt(m_CurrentPosition)->Play();
		}
		m_MusicList.GetAt(m_CurrentPosition)->SetVolume( AudioConfig::Get().GetVolume() );

		Lock.Unlock();
	}
}

/* -------------- */
/* PlayList::Stop */
/* -------------- */
void PlayList::Stop()
{
	Music *ptrMusic = GetCurrent();
	
	if ( ptrMusic != NULL )
	{
		ptrMusic->Stop();
		m_CurrentPosition = m_MusicList.GetHeadPosition();
	}
}

/* ------------------ */
/* PlayList::AddMusic */
/* ------------------ */
bool PlayList::AddMusic(const CString& _FileName)
{
	Music *tmpMusic = new Music();
	if ( tmpMusic->Load(_FileName) ) 
	{
		CSingleLock Lock( &AccessMusicList );
		Lock.Lock();

		m_MusicList.AddTail(tmpMusic);

		Lock.Unlock();
		return true;
	}
	delete tmpMusic;
	return false;
}

/* ------------------- */
/* PlayList::Serialize */
/* ------------------- */
void PlayList::Serialize(CArchive& _Archive)
{
	// Serialisation
	if ( _Archive.IsStoring() )
	{
		_Archive << m_MusicList.GetSize();
		
		if ( !m_MusicList.IsEmpty() )
		{
			POSITION it = m_MusicList.GetHeadPosition();
			_Archive << m_MusicList.GetAt(it)->GetFileName();
			while (it != m_MusicList.GetTailPosition())
			{
				m_MusicList.GetNext(it);
				_Archive << m_MusicList.GetAt(it)->GetFileName();
			}
		}
	}
	// Dserialisation
	else
	{
		ClearList();

		unsigned int MusicCount;
		_Archive >> MusicCount;
		
		for (unsigned int i = 0; i < MusicCount; i++)
		{
			CString FileName;
			_Archive >> FileName;
			Music *tmpMusic = new Music();
			if ( tmpMusic->Load(FileName) )
			{
				m_MusicList.AddTail(tmpMusic);
			}
		}

		m_CurrentPosition = m_MusicList.GetHeadPosition();
	}
}

/* ------------------- */
/* PlayList::ClearList */
/* ------------------- */
void PlayList::ClearList()
{
	POSITION it = m_MusicList.GetHeadPosition();
	while (it != m_MusicList.GetTailPosition())
	{
		m_MusicList.GetAt(it)->Stop();
		delete m_MusicList.GetAt(it);
		m_MusicList.GetNext(it);
	}

	m_MusicList.RemoveAll();
	m_CurrentPosition = NULL;
}

/* --------------------------- */
/* PlayList::GetRandomIterator */
/* --------------------------- */
POSITION PlayList::GetRandomIterator()
{
	POSITION Res;
	sf::Randomizer Rand;

	int Max = m_MusicList.GetSize() - 1;
	int NewIt = Rand.Random(0, Max);

	Res = m_MusicList.GetHeadPosition();
	for (int i = 0; i < NewIt; i++)
	{
		m_MusicList.GetNext(Res);
	}

	if ( ( Max > 0 ) && ( Res == m_CurrentPosition ) )
	{
		Res = GetRandomIterator();
	}

	return Res;
}

/* ---------------- */
/* PlayList::Update */
/* ---------------- */
void PlayList::Update()
{
	Music *ptrMusic = GetCurrent();
	
	if ( ptrMusic != NULL )
	{
		ptrMusic->SetVolume( AudioConfig::Get().GetVolume() );
	}
}

/* ---------------------- */
/* PlayList::GetMusicList */
/* ---------------------- */
CList<Music*, Music*>& PlayList::GetMusicList()
{
	return m_MusicList;
}

/* ---------------- */
/* PlayList::PlayAt */
/* ---------------- */
void PlayList::PlayAt(const POSITION& _Pos)
{
	Music *ptrMusic = GetCurrent();
	
	if ( ptrMusic != NULL )
	{
		ptrMusic->Stop();

		if ( ( ptrMusic = m_MusicList.GetAt(_Pos) ) != NULL )
		{
			m_CurrentPosition = _Pos;
			ptrMusic->Play();
		}
	}
}

/* ------------------ */
/* PlayList::RemoveAt */
/* ------------------ */
void PlayList::RemoveAt(const POSITION& _Pos)
{
	if ( _Pos == m_CurrentPosition )
	{
		Next();
	}

	Music *ptrMusic = m_MusicList.GetAt(_Pos);
	
	if ( ptrMusic != NULL )
	{
		CSingleLock Lock( &AccessMusicList );
		Lock.Lock();

		delete ptrMusic, ptrMusic = NULL;
		m_MusicList.RemoveAt(_Pos);

		Lock.Unlock();
	}
}

/* ---------------------------- */
/* PlayList::SetCurrentPosition */
/* ---------------------------- */
void PlayList::SetCurrentPosition(const POSITION& _Pos)
{
	m_CurrentPosition = _Pos;
}
