#pragma once

/**
* Class PlayList
* This class manages a play list. 
* @author Adrian Gaudebert - adrian.gaudebert@gmail.com
* @author Simon Stephan - s.stephan88@gmail.com
*/
class PlayList : public CObject, public Observer
{

private:	//--> Attributs privs

	CList<Music*, Music*> m_MusicList;

	POSITION m_CurrentPosition;

public:		//--> Attributs publics

	CMutex AccessMusicList;

private:	//--> Mthodes prives

	POSITION GetRandomIterator();

public:		//--> Mthodes publiques

	/**
	* Default constructor
	*/
	PlayList();

	/**
	* Destructor
	*/
	~PlayList();

	/**
	* Destroy all the musics and clear the list
	*/
	void ClearList();

	/**
	* Manage the current playing music, and call automaticaly the Next method
	*/
	void Manage();

	/**
	* Get the current playing music
	* @return Pointer to the music
	*/
	Music* GetCurrent();

	/**
	* Get he last music of the list
	* @return Pointer to the music
	*/
	Music* GetLast();

	/**
	* Go to the next music of the list
	* @param _Force If true (user clicked the Next button), goes to the first music if the current is the last
	*/
	void Next(bool _Force = false);

	/**
	* Go to the previous music
	*/
	void Previous();

	/**
	* Stop playing the current music
	*/
	void Stop();

	/**
	* Load a music and add to the end of the list
	*/
	bool AddMusic(const CString& _FileName);

	/**
	* Serialize the playlist in a CArchive. Used to save and load into and from files. 
	*/
	virtual void Serialize(CArchive& _Archive);

	/**
	* Update method, called by subjects when changes happen. 
	*/
	virtual void Update();

	/**
	* Get the music list object
	* @return CList containing the current list of musics
	*/
	CList<Music*, Music*>& GetMusicList();

	/**
	* Play the music at the given position
	* @param _Pos Position of the music
	*/
	void PlayAt(const POSITION& _Pos);

	/**
	* Remove a music at a given position
	@ param _Pos Position of the music
	*/
	void RemoveAt(const POSITION& _Pos);

	/**
	* Set the current playing music position
	* @param _Pos Position of the music
	*/
	void SetCurrentPosition(const POSITION& _Pos);

	//--> For serialisation
	DECLARE_SERIAL(PlayList);

};
